<?php
// Author: Alain D D Williams <addw@phcomp.co.uk>
// SCCS: @(#)FirstTimeConfig.php	1.11 06/20/11 12:47:10
// Copyright (C) the Author 2005 - 2011.

// Call this at the start of every script, it only does something once (if $ConfigArray[Something] is not set)
// This reads basic config info into $ConfigArray - so it will be shared between apps that use the same config.
// Different apps (with clashing config param names) can be used in the same session as long as $app is different
// iff $multiConf is TRUE.
// Subsequent calls add new conf info if needs be.
// $app is an application name that (lowercased) maps onto a conf file.
// $app is stored in global $ConfigAppName (not lowercased)
// The conffile used will be kept in $ConfigArray['_ConfFile'] & used on subsequent calls.
// $ConfigArray['_ConfRelDir'] holds the directory that the conffile may be relative to.
// Return false if nothing new value was got.
// Return the config root if got something, this can be passed to ReadConfigValue().
// Set $multiConf if you expect more than one app config under the web site & so get vars from $ConfigArray
// instead of $_SESSION. $ConfigArray will be referenced to $_SESSION[$ConfigAppName]
function FirstTimeConfig($app, $ConfInfo = array('LogFile', 'AppName'), $multiConf = false)
{
    global $ConfigAppName;

    $ConfigAppName = $app;

    // Beware how global works if a var is not defined. Need to create if $GLOBALS first
    if($multiConf) {
        if( !isset($_SESSION[$ConfigAppName]))
            $_SESSION[$ConfigAppName] = array();
        $GLOBALS['ConfigArray'] = &$_SESSION[$ConfigAppName];
    } else {
        $GLOBALS['ConfigArray'] = &$_SESSION;
    }

    global $ConfigArray;    // must be after the above

    $confFile = '_ConfFile';
    $confRelDir = '_ConfRelDir';

    // Do we have everything that we need ?
    $GetConf = array();
    foreach($ConfInfo as $conf)
        if( !isset($ConfigArray[$conf]))
            $GetConf[] = $conf;

    // Need to determine a config file ? We might have it from last time, but invalidate it if we changed directory
    if(isset($ConfigArray[$confFile]) && ( !isset($ConfigArray[$confRelDir]) || $ConfigArray[$confRelDir] !== dirname($_SERVER['SCRIPT_FILENAME'])))
        unset($ConfigArray[$confFile]);

    if( !$GetConf && isset($ConfigArray[$confFile]))    // Know it all!
        return false;

    // Generate a search path of directories to look in:
    // . .. document_root/.. include_path /etc
    $pathArray = array('.', '..');
    if(isset($_SERVER["DOCUMENT_ROOT"]))
        array_push($pathArray, dirname($_SERVER["DOCUMENT_ROOT"]));
    $pathArray = array_unique(array_merge($pathArray, explode(PATH_SEPARATOR, get_include_path()), array('/etc')));

    if( !isset($ConfigArray[$confFile])) {
        // Don't do a mapping if the AppName contains a '/'
        if(strpos($app, '/') === FALSE) {
            // Since this program can be used by various applications, all of which will have different
            // configs, look for a appname .confmap to tell us what the real config file is.
            // Take the first alphanumeric word in the file, this is then used to find a word.conf in $pathArray
            $conf = '/' . Strtolower($app) . '.confmap';
            foreach($pathArray as $ConfFile) {
                $ConfFile .= $conf;
		// Need @ below to avoid error messages if have open_basedir restriction
                if(@is_readable($ConfFile) && ($cont = @file_get_contents($ConfFile)) !== FALSE &&
                        preg_match('/^(\w+)/', $cont, $match)) {
                    $app = $match[1];
                    break;
                }
            }
        }

        // If the (possibly new) app contains a '/' - don't search, just check that it exists
        if(strpos($app, '/') !== FALSE) {
            if( file_exists($ConfFile)) {
                $ConfigArray[$confFile] = $ConfFile = $app;
            }
        } else {
            // Search:
            foreach($pathArray as $ConfDir) {
                $ConfFile = $ConfDir . '/' . $app . '.conf';
                if( file_exists($ConfFile)) {
                    $ConfigArray[$confFile] = $ConfFile;
                    break;
                }
            }
        }

        if( !isset($ConfigArray[$confFile]))
            die("Cannot find a configuration file for $app\n");

        $ConfigArray[$confRelDir] = dirname($_SERVER['SCRIPT_FILENAME']);
    }

    require_once "PHlib/ReadConfig.php";
    $ConfigRoot = ReadConfig($GetConf, $ConfigArray[$confFile]);

    return $ConfigRoot;
}
