<?php
// Author: Alain D D Williams <addw@phcomp.co.uk>
// SCCS: @(#)LogMsg.php	1.9 08/04/11 18:37:00
// Copyright (C) the Author 2005 - 2011.

// Log a message
// opts (each as a string):
// -1	To the web page being generated, or stdout if CLI
// -2	system log
// -A	error log - PEAR_LOG_ALERT
// -e	error log - PEAR_LOG_ERR
// -E	error log - PEAR_LOG_EMERG and to the web page
// -W	error log - PEAR_LOG_WARNING
// -N	error log - PEAR_LOG_NOTICE
// -I	error log - PEAR_LOG_INFO
// -D	error log - PEAR_LOG_DEBUG
// -a	All: -1 -2 -e
// -f	append name of calling function/file/line_number
// -f n	as -f but not the calling function, but 'n' calling.
// -t   Append function backtrace - replaces any value to -f; could be quite long
// -Q	Quit after writing message
// This will kill the program (die) if it detects a call loop or
// $ConfigArray['LogFile'] is not available.

function LogMsg() {
    global $ConfigArray;

    static $Called = 0;    // detect/prevent infinite recursion

    require_once 'Log.php';

    // Init options:
    $syslog = $errlog = 0; $browser = 0; $calling = 0; $msgonly = $backtrace = 0;
    $LogLevel = PEAR_LOG_ERR; $quit = false;

    $trace = debug_backtrace();
    @ $calledStr = " Called at {$trace[1]["file"]}:{$trace[1]["line"]}"; // @ because sometimes don't get it

    $args = func_get_args();
    while(isset($args[0]) && substr($args[0], 0, 1) == '-') {
        $arg = array_shift($args);
        switch($arg) {
        case '-1':    // 'stdout' - ie web_page/browser
            $browser = 1;
            break;
        case '-2':    // system log
            $syslog = 1;
            break;
        case '-e':    // errorlog
            $errlog = 1; $LogLevel = PEAR_LOG_ERR;
            break;
        case '-A':    // errorlog
            $errlog = 1; $LogLevel = PEAR_LOG_ALERT;
            break;
        case '-E':    // errorlog
            $browser = 1; $errlog = 1; $LogLevel = PEAR_LOG_EMERG;
            break;
        case '-W':    // errorlog
            $errlog = 1; $LogLevel = PEAR_LOG_WARNING;
            break;
        case '-N':    // errorlog
            $errlog = 1; $LogLevel = PEAR_LOG_NOTICE;
            break;
        case '-I':    // errorlog
            $errlog = 1; $LogLevel = PEAR_LOG_INFO;
            break;
        case '-D':    // errorlog
            $errlog = 1; $LogLevel = PEAR_LOG_DEBUG;
            break;
        case '-a':    // All
            $syslog = $browser = $errlog = 1; $LogLevel = PEAR_LOG_ERR;
            break;
        case '-f':
            $calling = 1;
            break;
        case '-t':    // Backtrace
            $backtrace = 1;
            break;
        case '-Q':    // Quit
            $quit = true;
            break;
        default:    // Deal with '-f n'
            if(preg_match('/^-f\s*(\d+)$/', $arg, $match)) {
                if(($calling = $match[1]) < count($trace))
                    $calledStr = " Called at ". (isset($trace[$calling]["file"]) ? $trace[$calling]["file"] : 'unknown file') . ":" .
                        (isset($trace[$calling]["line"]) ? $trace[$calling]["line"] : 'unknown line');
                break;
            }
            LogMsg('-2', '-f', "Unknown LogMsg option '%s'.", $arg);
        }
    }

    // The rest is the message
    $fmt = array_shift($args);
    $Message = vsprintf($fmt, $args);

    // Generate a call trace ?
    if($backtrace) {
        array_shift($trace);    // Remove reference to LogMsg()
        $calledStr = " Backtrace";
        $file = '';
        foreach($trace as $func) {
            // Note that things like file & line are not always set!
            if(isset($func['file'])) {
                $calledStr .= ' ';

                // Last component of path, if different from last time
                $nfile = substr(strrchr($func['file'], "/"), 1);
                if($file !== $nfile) {
                    $calledStr .= $nfile;
                    $file = $nfile;
                }

                $calledStr .= ':' . (isset($func['line']) ? $func['line'] : '?');
                $calledStr .= " {$func['function']}()";
            }
       }
    }

    if($Called++ > 3) {	// Trap deep recursion
        error_log("LogMsg - function infinitely called, '$fmt' $calledStr", 0);
        die("Application error: LogMsg function recursion error\n");
    }

    if($calling || $backtrace)
        $Message .= $calledStr;

    // Output to the various places, simplest first, browser or stdout:
    if($browser) {
        // If command line, just append a newline:
        if(php_sapi_name() == 'cli' && !isset($_SERVER['REMOTE_ADDR'])) {
            echo $Message . "\n";
        } else {
            // To a browser:
            $break = $strong = $unstrong = '';
            if(ini_get('html_errors')) {
                $break = '<br>';
                if($LogLevel == PEAR_LOG_ERR || $LogLevel == PEAR_LOG_ALERT) {
                    $strong = '<strong>'; $unstrong = '</strong>';
                }
            }

            echo "\n$break$strong" . htmlspecialchars($Message) . " $unstrong$break\n";
        }
    }

    // stderr or apache log:
    if($syslog)
        error_log($Message, 0);

    // PEAR logging:
    if($errlog) {
        static $Log;

        if( ! isset($Log)) {
            global $ConfigArray;
            if( ! isset($ConfigArray['LogFile']))
                LogMsg('-2', '-f', "Config var ConfigArray['LogFile'] is not available.");
            else
                $Log = Log::factory('file', $ConfigArray['LogFile']);
        }

        if(isset($Log))
            $Log->log($Message, $LogLevel);
    }

    if($quit)
        exit(2);

    $Called--;
}
