<?php
// Read config and setup a few important config values into $_SESSION
// If a var cannot be obtained, the program is aborted.
// Author: Alain D D Williams <addw@phcomp.co.uk>
// SCCS: @(#)ReadConfig.php	1.6 06/20/11 12:47:12
// Copyright (C) the Author 2005 - 2011.

require_once "Config.php";

// Log to $ConfigArray['LogFile'] - which should be the first value obtained from config.

// Read conf values into $ConfigArray (default $_SESSION[$GLOBALS['ConfigAppName']] or $_SESSION), return the config handle that we can use to get other values
function ReadConfig($ConfNames, $ConfFile)
{
        // Beware how global works if a var is not defined. Need to create if $GLOBALS first
        // Try to guess where the config is
        if( !isset($GLOBALS['ConfigArray'])) {
            if(isset($GLOBALS['ConfigAppName']) && isset($_SESSION[$GLOBALS['ConfigAppName']]))
                $GLOBALS['ConfigArray'] = &$_SESSION[$GLOBALS['ConfigAppName']];
            else
                $GLOBALS['ConfigArray'] = &$_SESSION;
        }

	global $ConfigArray;

	$conf = new Config();
	$root = $conf->parseConfig($ConfFile, 'GenericConf');

	if(PEAR::isError($root))
		LogMsg('-2', '-f', '-Q', "Whooops - can't get configuration from '$ConfFile': '%s'", $root->getMessage());

	// Get & set the values in ConfNames into $ConfigArray:
	foreach($ConfNames as $Name)
		$ConfigArray[$Name] = ReadConfigValue($Name, $root);

	return($root);
}

// Get a value from the config $root
function ReadConfigValue($Name, $root, $NotExistOK = TRUE) {
	if(($val = $root->directiveContent($Name)) !== FALSE)
		return($val);

        if( $NotExistOK )
            return(FALSE);

	require_once "PHlib/LogMsg.php";
	LogMsg('-2', '-f2', '-Q', "Aborting due to configuration error, missing directive: '%s'", $Name);
}

// Get named config values as an array, key:value_name value:value.
// $varnames - string of space separated names to get from config and to return in an array.
//       Each name may be of the form arrayKey:configVarname
// $root - a Config class as returned from FirstTimeConfig(), if FALSE then the
//         values should be cached in the session
// $confName name of the array in the ConfigArray where these values are cached.
// Return the config values array, or an empty array.
// Stored in global array $ConfigArray
function ConfigReadVarList($varnames, $root, $confName)
{
    global $ConfigArray;

    // If we havent set this up and have a config then set it up:
    if( ! isset($ConfigArray[$confName]) && $root !== FALSE) {
        $vars = array();
        foreach(explode(' ', $varnames) as $var) {
            if($var == ' ')    // be kind if multiple spaces
                continue;

            // Separate out value and config names, if different:
            if(($pos = strpos($var, ':')) === FALSE)
                $cname = $sname = $var;
            else {
                $sname = substr($var, 0, $pos);
                $cname = substr($var, $pos + 1);
            }

            $vars[$sname] = ReadConfigValue($cname, $root);
        }
        if(count($vars))    // Find something ? Store in session:
            $ConfigArray[$confName] = $vars;
    }

    return isset($ConfigArray[$confName]) ? $ConfigArray[$confName] : array();
}
