<?php
// Misc functions to help in the use of Smarty and QuickForm2.
// SCCS: @(#)SmartyDisp.php	1.10 09/28/11 23:00:55
// Copyright Alain D D Williams <addw@phcomp.co.uk>, 2010

// Global variables:
// $smarty    - the Smarty object

// Expected values in $ConfigArray, if not there they are not used, default is
// relative to the current directory - use this if you have a multi template directory site:
// SmartyTemplateRoot      The directory in which the templates are found
// SmartyCompiledTemplate  The directory in which the compiled templates are put

// Bring in the smarty code & assign to $smarty
// $init is an optional array of smarty variables to be assigned: key/value
function SmartyStart($init = array())
{
    global $smarty, $ConfigArray;

    if(isset($smarty))
        return;

    require_once 'Smarty.class.php';
    $smarty = new Smarty;

    if(isset($ConfigArray['SmartyTemplateRoot']))
        $smarty->template_dir = $ConfigArray['SmartyTemplateRoot'];
    if(isset($ConfigArray['SmartyCompiledTemplate']))
        $smarty->compile_dir  = $ConfigArray['SmartyCompiledTemplate'];

    $smarty->compile_check = true;
    $smarty->caching = 0;

    foreach($init as $key => $val)
        $smarty->assign($key, $val);
}

// Display the template and exit the program
// $template  The name of the smarty template file, will have .tpl appended
// Before display call SmartyStdVars() if it exists.
function SmartyDisplayTemplate($template)
{
    global $smarty;

    if(function_exists('SmartyStdVars'))
        SmartyStdVars();

    $smarty->display($template . '.tpl');

    exit;
}

// Show where we are - here, for trace/debug. Display DevPlaceholder.tpl and return
// $where - where we are, assign to 'Where'
// $show - array of values, assign to 'Show'
// Before display call SmartyStdVars() if it exists.
function SmartyShowHere($where, $show = NULL)
{
    global $smarty;

    $smarty->assign('Where', $where);
    $smarty->assign('Show', print_r($show, true));

    if(function_exists('SmartyStdVars'))
        SmartyStdVars();

    $smarty->display('DevPlaceholder.tpl');
}

// Add values from $_SESSION to smarty
// $vars is an array where the keys are the names of session/smarty variables,
// values are true if this must exist (isset()), false if it doesn't matter.
// On fail SmartyDisplayError('NotInSession') template and quit.
// $where is where we are called from, used as 2nd arg to SmartyDisplayError()
// Depricated - SmartyFromConfig()
function SmartyFromSession($vars, $where)
{
    global $smarty;

    foreach($vars as $var => $must) {
        if(isset($_SESSION[$var])) {
            $smarty->assign($var, $_SESSION[$var]);
            continue;
        }

        if( ! $must)
            continue;

        // Whoops - something horrible
        SmartyDisplayError('-f2', 'NotInSession', "$where, unset $var");
    }
}

// Depricated, use SmartyFromSession() which does the same thing.
function SessionToSmarty($vars, $where)
{
    SmartyFromSession($vars, $where);
}

// As SmartyFromSession(), but from $ConfigArray.
// On fail SmartyDisplayError('NotInConfig') template and quit.
function SmartyFromConfig($vars, $where)
{
    global $smarty, $ConfigArray;

    foreach($vars as $var => $must) {
        if(isset($ConfigArray[$var])) {
            $smarty->assign($var, $ConfigArray[$var]);
            continue;
        }

        if( ! $must)
            continue;

        // Whoops - something horrible
        SmartyDisplayError('-f2', 'NotInConfig', "$where, unset $var");
    }
}

// Set some smarty variables with values from the config file.
// The values are cached in the session as an array named 'Smarty_XX' where 'XX' is the config name.
// $varnames - string of space separated names to get from config and to set as smarty variables.
//       Each name may be of the form smartyVarname:configVarname
// $root - a Config class as returned from FirstTimeConfig(), if FALSE then the
//         values should be cached in the session
function SmartyVarsFromConfig($varnames, $root)
{
    require_once 'PHlib/ReadConfig.php';

    global $smarty;

    $smarty->assign(ConfigReadVarList($varnames, $root, 'Smarty'));
}

// This records an error with LogMsg(), displays an error page with SmartyDisplayTemplate() and quits.
// Initial args starting with '-' are passed to LogMsg() (defaults: -e -f2), except for:
// -U word   Next arg word is assigned to smarty variable 'UserErr' and so is shown to the user and is logged.
// This must be followed by 2 args:
// PageName is the template name to display (with .tpl appended) and is used in logging
// ErrValue is something that is logged but not displayed, it will be made safe for logging
function SmartyDisplayError()
{
    global $smarty;

    if( ! function_exists('LogMsg'))
        require_once 'PHlib/LogMsg.php';

    $lm_args = array('-e', '-f2');    // Options to LogMsg

    $userErr = '';

    // Process/gather options:
    $args = func_get_args();
    while(isset($args[0]) && substr($args[0], 0, 1) == '-') {
        $a = array_shift($args);
        if($a == '-U') {
            if(isset($args[0])) {
                $userErr = array_shift($args);
                $smarty->assign('UserErr', $userErr);
            }
        } else
            $lm_args[] = $a;
    }

    if(count($args) != 2)    // Sanity check
        LogMsg('-e', '-f2', "SmartyDisplayError() bad number of arguments");

    $PageName = array_shift($args);
    $ErrValue = array_shift($args);

    array_push($lm_args, "Error %s.%s", $PageName, addcslashes(($userErr == '' ? '' : (' UserErr' . $userErr . '.')) . ' ErrValue: ' . $ErrValue, "\0..\37\177..\377"));
    call_user_func_array('LogMsg', $lm_args);

    SmartyDisplayTemplate($PageName);
}

// Display a form using smarty and exit the program
// $form      a HTML_QuickForm2 object
// $template  The name of the smarty template file, will have .tpl appended
// $group_errors Set to true if field errors to be grouped, ie at the end of the form, default false so next to fields
function SmartyDisplayForm($form, $template, $group_errors = false)
{
    global $smarty;

    require_once 'HTML/QuickForm2/Renderer.php';

    // Create the renderer object
    HTML_QuickForm2_Renderer::register('smarty', 'HTML_QuickForm2_Renderer_Smarty');
    $renderer = HTML_QuickForm2_Renderer::factory('smarty');
    if($group_errors)
        $renderer->setOption('group_errors', true);

    // build the HTML for the form
    $FormData = $form->render($renderer)->toArray();

    // assign array with form data
    $smarty->assign('FormData', $FormData);

    SmartyDisplayTemplate($template);
}

// Render a form using smarty and return the rendered array
// Use this if there are to be several forms on one page
// $form      a HTML_QuickForm2 object
// $group_errors Set to true if field errors to be grouped, ie at the end of the form, default false so next to fields
function SmartyFormRender($form, $group_errors = false)
{
    global $smarty;
 
    require_once 'HTML/QuickForm2/Renderer.php';
 
    // Create the renderer object
    try {
        HTML_QuickForm2_Renderer::register('smarty', 'HTML_QuickForm2_Renderer_Smarty');
    } catch(HTML_QuickForm2_InvalidArgumentException $e) {
        ;   // The renderer might already be registered ... must be a better way of doing this! There is not a method to check
    }
    $renderer = HTML_QuickForm2_Renderer::factory('smarty');
    if($group_errors)
        $renderer->setOption('group_errors', true);
    
    // build the Array from the form
    return($form->render($renderer)->toArray());
}

// Check that the $_GET array contains what it should, SmartyDisplayError('BadGetArg') if not.
// $chklist is an array:
// key   GET field
// value RE to check it
// $where is where we are called from
function SmartyCheckGetArgs($chklist, $where)
{
    foreach($chklist as $arg => $re) {
        if(isset($_GET[$arg]) &&
            preg_match($re, $_GET[$arg]))
                continue;

        SmartyDisplayError('-f3', 'BadGetArg', "$where, arg=$arg, val='" . (isset($_GET[$arg]) ? $_GET[$arg] : 'UNSET') . "'");
    }
}

// end
